// Example for logical decoding (PostgreSQL 14)
// Copyright (c) 2021 by lutz Froehlich
import java.nio.ByteBuffer;
import java.sql.*;
import java.util.Properties;
import java.util.concurrent.TimeUnit;
import org.postgresql.PGConnection;
import org.postgresql.PGProperty;
import org.postgresql.replication.PGReplicationStream;

public class PGLogicalDecoding {
  public PGLogicalDecoding() {
  }

  private static PGConnection getDBConnection(String username, String password) {
    PGConnection replcon = null;
    String url = "jdbc:postgresql://localhost/postgres";

    Properties props = new Properties();
    PGProperty.USER.set(props, "Lutz");
    PGProperty.PASSWORD.set(props, "friseuse");
    PGProperty.ASSUME_MIN_SERVER_VERSION.set(props, "9.4");
    PGProperty.REPLICATION.set(props, "database");
    PGProperty.PREFER_QUERY_MODE.set(props, "simple");

    Connection conn = null;

    try {
      conn = DriverManager.getConnection(url, props);
      replcon = conn.unwrap(PGConnection.class);
    } catch (SQLException sex) {
      sex.printStackTrace();
    }
    return replcon;
  }

  private static String execSQL(Connection conn, String sqlstmt) {
    String retval = null;
    try {
      Statement stmt = conn.createStatement();
      ResultSet rset = stmt.executeQuery(sqlstmt);
      if (rset.next()) {
        retval = rset.getString(1);
      } else {
        retval = "No recoord found...";
      }
    } catch (SQLException sex) {
      sex.printStackTrace();
    }
    return retval;
  }

  public static void main(String[] args) {
    System.out.println("Test output from logical decoding");
    System.out.println("Copyright (c) 2021 by Lutz Froehlich");
    PGConnection repcon = getDBConnection("Lutz", "friseuse");
    try {
      repcon.getReplicationAPI().createReplicationSlot().logical().withSlotName("hanser_logical_slot").withOutputPlugin("test_decoding").make();

      PGReplicationStream stream = repcon.getReplicationAPI().replicationStream().logical().withSlotName("hanser_logical_slot").withSlotOption("include-xids", false).withSlotOption("skip-empty-xacts", true).withStatusInterval(20, TimeUnit.SECONDS).start();
      while (true) {
        ByteBuffer msg = stream.readPending();
        if (msg == null) {
          try {
            TimeUnit.MICROSECONDS.sleep(10L);
          } catch (InterruptedException iex) {
            iex.printStackTrace();
          }
          continue;
        }
        int offset = msg.arrayOffset();
        byte[] source = msg.array();
        int length = source.length - offset;
        System.out.println(new String(source, offset, length));
        //feedback
        stream.setAppliedLSN(stream.getLastReceiveLSN());
        stream.setFlushedLSN(stream.getLastReceiveLSN());
      }
    } catch (SQLException sex) {
      sex.printStackTrace();
    }
  }
}
